<?php
/**************************************************
  Coppermine Plugin - Sort by EXIF
  *************************************************
  Copyright (c) 2018 Ludger Hellerhoff
  *************************************************
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3
  as published by the Free Software Foundation.
***************************************************/

// if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

require_once $workdir.'/plugins/sort_by_exif/pel-master/autoload.php';

use lsolesen\pel\Pel;
use lsolesen\pel\PelTag;
use lsolesen\pel\PelEntryAscii;
use lsolesen\pel\PelExif;
use lsolesen\pel\PelIfd;
use lsolesen\pel\PelEntryTime;
use lsolesen\pel\PelDataWindow;
use lsolesen\pel\PelJpeg;
use lsolesen\pel\PelTiff;

function has_exif_meta($input) {
	return(set_exif_meta($input, null, null, 1));
};

function get_exif_meta($input) {
	return(set_exif_meta($input, null, null, 0, 1));
};


function set_exif_meta($input, $creation_time = null, $digitized_time = null, $justcheck = 0, $justlook = 0) {
global $workdir;
$result = array();
if (file_exists($input)) {
	//ini_set('memory_limit', '64M');
	$data = new PelDataWindow(file_get_contents($input));
	if (PelJpeg::isValid($data)) {
		$jpeg = $file = new PelJpeg();
		$jpeg->load($data);
		$exif = $jpeg->getExif();
		if ($justcheck) {return(($exif == null) ? 0 : 1);};
		if ($exif == null) {
			$exif = new PelExif();
			$jpeg->setExif($exif);
			$tiff = new PelTiff();
			$exif->setTiff($tiff);
		} else {
			$tiff = $exif->getTiff();
		}
	} elseif (PelTiff::isValid($data)) {
		$tiff = $file = new PelTiff();
		$tiff->load($data);
	} else {
		$result['error'] = 'No valid Data';
		return($result);
	}
	$ifd0 = $tiff->getIfd();
	if ($ifd0 == null) {
		$ifd0 = new PelIfd(PelIfd::IFD0);
		$tiff->setIfd($ifd0);
	}
	$exif = $ifd0->getSubIfd(PelIfd::EXIF);
	if ($exif == null) {
		$exif  = new PelIfd(PelIfd::EXIF);
		$ifd0->AddSubIfd($exif, PelIfd::EXIF);
	}	
	
	$cdate = $exif->getEntry(PelTag::DATE_TIME_ORIGINAL);
	$ddate = $exif->getEntry(PelTag::DATE_TIME_DIGITIZED);	
	if ($justlook) {
		$result['creation_time'] = $cdate->getValue();
		if (isset($result['creation_time'])) {$result['creation_time'] = date('Y-m-d H:i:s', $result['creation_time']);};
		$result['digitized_time'] = $ddate->getValue();
		if (isset($result['digitized_time'])) {$result['digitized_time'] = date('Y-m-d H:i:s', $result['digitized_time']);};
		return $result;
	};

	if ($creation_time <> null) {
		if ($cdate == null) {
			$cdate = new PelEntryTime(PelTag::DATE_TIME_ORIGINAL, strtotime($creation_time));
			$exif->addEntry($cdate);
		} else {
			$cdate->setValue(strtotime($creation_time));};
	};
	if ($digitized_time <> null) {
		if ($ddate == null) {
			$ddate = new PelEntryTime(PelTag::DATE_TIME_DIGITIZED, strtotime($digitized_time));
			$exif->addEntry($ddate);
		} else {$ddate->setValue(strtotime($digitized_time));};
	};
	$tmpfname = tempnam($workdir.'/plugins/sort_by_exif/temp', "SBE");	
	$file->saveFile($tmpfname);
	unlink($input);
	rename($tmpfname,$input);
	chmod($input,0644);
} else  {$result['error'] = 'File not found.';};
return $result;
};

?>